#!/usr/bin/python-init -Otu
#
# Copyright (C) 2008,2009,2012 Andreas Lang-Nevyjel
#
# Send feedback to: <lang-nevyjel@init.at>
#
# This file is part of python-modules-rrd
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
""" rrdtool dumper and merger """

import optparse
import os
import re
import sys
import tarfile
import cPickle
import pprint
import mail_tools
import time
import commands
import rrdtool
import pprint
import configfile
import bz2
import tempfile
import logging_tools
import process_tools
import pwd
import copy
import rrd_tools

class my_opt_parser(optparse.OptionParser):
    def __init__(self):
        optparse.OptionParser.__init__(self)
        self.add_option("--dump-base-dir", dest="dump_base_dir", help="set base directory for dump, [%default]", default="/var/lib/rrddump")
        self.add_option("--src", dest="source_file", help="file(s) to read [%default]", default="")
        self.add_option("--dst", dest="dest_file", help="file to write [%default]", default="")
        self.add_option("--ts-list", dest="timestamp_list", help="coma-separated timestamplist if more than one src-file is specified", default="")
        self.add_option("--merge", dest="merge_file", help="file to merge intelligently [%default]", default="")
        self.add_option("--copy-inline", dest="copy_inline", help="copies data from one rra to the other [%default]", action="store_true", default=False)
        self.add_option("--only-info", dest="only_info", help="show only info [%default]", default=False, action="store_true")
        self.add_option("--no-restore", dest="disable_restore", help="disables 'rrdtool restore' command for created file [%default]", default=False, action="store_true")
        self.add_option("--uni-fix", dest="uni_fix", action="store_true", default=False, help="fixes the filepermission for SELinux")
    def parse(self):
        options, args = self.parse_args()
        if args:
            print "Additional arguments found, exiting"
            sys.exit(0)
        self.options = options

def main():
    g_config = configfile.configuration("wri", {"LOG_NAME"        : configfile.str_c_var("rrddump"),
                                                "LOG_DESTINATION" : configfile.str_c_var("uds:/var/lib/logging-server/py_log")})
    my_parser = my_opt_parser()
    my_parser.parse()
    g_config.add_config_dict({"MAIN_DIR"        : configfile.str_c_var(my_parser.options.dump_base_dir),
                              "SOURCE_FILES"    : configfile.array_c_var([name.strip() for name in my_parser.options.source_file.split(",") if name.strip()]),
                              "TS_LIST"         : configfile.array_c_var([int(ts.strip()) for ts in my_parser.options.timestamp_list.split(",") if ts.strip()]),
                              "DEST_FILE"       : configfile.str_c_var(my_parser.options.dest_file),
                              "MERGE_FILE"      : configfile.str_c_var(my_parser.options.merge_file),
                              "DISABLE_RESTORE" : configfile.bool_c_var(my_parser.options.disable_restore),
                              "PERM_FIX"        : configfile.bool_c_var(my_parser.options.uni_fix)})
    if len(g_config["SOURCE_FILES"]) - len(g_config["TS_LIST"]) != 1:
        print "Error: number of timestamp must be equal to number of source-files minus one (is now %d and %d)" % (len(g_config["TS_LIST"]),
                                                                                                                   len(g_config["SOURCE_FILES"]))
        sys.exit(-1)
    # read all rra_objs
    rra_objs = []
    for src_file in g_config["SOURCE_FILES"]:
        if os.path.isfile(src_file):
            rra_obj = rrd_tools.rrd_rra(g_config, source_file=src_file)
            rra_obj.show_info(full_info=my_parser.options.only_info)
            if my_parser.options.copy_inline:
                rra_obj.copy_inline()
            rra_objs.append(rra_obj)
        else:
            print "Source file '%s' does not exist, exiting ..." % (src_file)
            sys.exit(-1)
    if my_parser.options.only_info:
        sys.exit(0)
    # merge according to ts_list
    if g_config["TS_LIST"]:
        print "Merging %s according to ts_list %s" % (logging_tools.get_plural("rra", len(rra_objs)),
                                                      ":".join(["%d" % (ts) for ts in g_config["TS_LIST"]]))
    if len(rra_objs) > 1:
        s_time, max_time = (0, 9999999999)
        last_rra = rra_objs[-1]
        for rra_index, rra_obj, end_time in zip(xrange(len(rra_objs)), rra_objs, g_config["TS_LIST"] + [max_time]):
            print "rra_obj #%d valid from %s to %s" % (rra_index,
                                                       time.ctime(s_time) if s_time else "-inf",
                                                       time.ctime(end_time) if end_time != max_time else "now")
            if end_time != max_time:
                # merge actual with last
                last_rra.copy_object(rra_obj, s_time, end_time)
            s_time = end_time
        rra_objs = [last_rra]
    rra_obj = rra_objs[0]
    if os.path.isfile(g_config["MERGE_FILE"]):
        merge_obj = rrd_tools.rrd_rra(g_config, source_file=g_config["MERGE_FILE"])
        merge_obj.show_info()
        rra_obj.merge_object(merge_obj)
    #print rra_obj["ds"]
    #print rra_obj["rra_list"]
    if g_config["DEST_FILE"]:
        if [True for ending in ["rra", "rrd"] if g_config["DEST_FILE"].endswith(ending)]:
            rra_obj.write_rra(g_config["DEST_FILE"], g_config["DISABLE_RESTORE"])
        else:
            rra_obj.dump(g_config["DEST_FILE"])

if __name__ == "__main__":
    main()
