#!/usr/bin/python-init -Otu
#
# Copyright (C) 2008,2009 Andreas Lang-Nevyjel
#
# Send feedback to: <lang-nevyjel@init.at>
#
# This file is part of python-modules-rrd
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
""" rrdtool dumper and merger """

import optparse
import os
import re
import sys
import tarfile
import cPickle
import pprint
import mail_tools
import time
import commands
import rrdtool
import pprint
import configfile
import bz2
import tempfile
import logging_tools
import process_tools
import pwd
import copy
import rrd_tools
import re

class my_opt_parser(optparse.OptionParser):
    def __init__(self):
        optparse.OptionParser.__init__(self)
        self.add_option("--modify", dest="modify", action="store_true", help="modify files [%default]", default=False)
        self.add_option("--info", dest="info", action="store_true", help="show info [%default]", default=False)
        self.add_option("--remove-rra", dest="rra_remove", type="str", help="RRAs to remove [%default]", default="")
        self.add_option("--add-rra", dest="rra_add", type="str", help="RRAs to add [%default]", default="")
        self.add_option("--copy-inline", dest="copy_inline", help="copies data from one rra to the other [%default]", action="store_true", default=False)
        self.add_option("--no-restore", dest="disable_restore", help="disables 'rrdtool restore' command for created file [%default]", default=False, action="store_true")
        self.add_option("--uni-fix", dest="uni_fix", action="store_false", default=True, help="fixes the filepermission for SELinux [%default]")
        self.add_option("--check-values", dest="check_values", action="store_true", default=False, help="check values of RRD [%default]")
        self.add_option("--min-value", dest="min_value", type="int", default=0, help="minimum value to accept [%default]")
        self.add_option("--max-value", dest="max_value", type="int", default=1000000000000, help="maximum value to accept [%default]")
        self.add_option("--cap-to-none", dest="cap_to_none", action="store_true", default=False, help="replaces out-of-bound values with None [%default]")

def main():
    options, args = my_opt_parser().parse_args()
    if not args:
        print "Need RRDs to operate on"
        sys.exit(0)
    options.rra_remove = [part.strip() for part in options.rra_remove.split(",") if part.strip()]
    options.rra_add = [part.strip() for part in options.rra_add.split(",") if part.strip()]
    # handle special case
    new_rra_add = []
    for rra_add in options.rra_add:
        if rra_add.startswith("RRA-*-"):
            new_rra_add.extend(["RRA-%s-%s" % (add_cf, rra_add[6:]) for add_cf in ["AVERAGE", "LAST", "MIN", "MAX"]])
        else:
            new_rra_add.append(rra_add)
    options.rra_add = new_rra_add
    if options.rra_remove:
        print "removing %s: %s" % (logging_tools.get_plural("RRA", len(options.rra_remove)),
                                   ", ".join(sorted(options.rra_remove)))
    if options.rra_add:
        print "adding %s: %s" % (logging_tools.get_plural("RRA", len(options.rra_add)),
                                 ", ".join(sorted(options.rra_add)))
    rra_remove_re = [re.compile("^%s$" % (part.replace("*", ".*"))) for part in options.rra_remove]
    for rrd_name in args:
        print "Operating on file %s" % (rrd_name)
        rrd_obj = rrd_tools.rrd_rra({"PERM_FIX" : options.uni_fix}, source_file=rrd_name)
        if options.info:
            rrd_obj.show_info(full_info=True)
        changed = False
        rras_to_remove = [rra_name for rra_name in rrd_obj["rra_list"] if len([True for re_obj in rra_remove_re if re_obj.match(rra_name)])]
        if rras_to_remove:
            changed = True
            print "removing %s: %s" % (logging_tools.get_plural("RRA", len(rras_to_remove)),
                                       ", ".join(sorted(rras_to_remove)))
            [rrd_obj.remove_rra(rra_name) for rra_name in rras_to_remove]
        rras_to_add = [rra_name for rra_name in options.rra_add if rra_name not in rrd_obj["rra_list"]]
        if rras_to_add:
            changed = True
            print "adding %s: %s" % (logging_tools.get_plural("RRA", len(rras_to_add)),
                                     ", ".join(sorted(rras_to_add)))
            ref_rra = rrd_obj["rra_dict"][rrd_obj["rra_list"][0]]
            [rrd_obj.add_rra(rrd_tools.create_rra(string_spec=rra_name,
                                                  ref_rra=ref_rra)) for rra_name in rras_to_add]
        if options.check_values:
            print "Checking values against [%s, %s]" % (str(options.min_value),
                                                        str(options.max_value))
            num_changed = rrd_obj.check_values(options.cap_to_none, options.min_value, options.max_value)
            if num_changed:
                changed = True
        if options.copy_inline:
            if rrd_obj.copy_inline():
                changed = True
        if changed and options.info:
            rrd_obj.show_info(full_info=True)
        if options.modify and changed:
            rrd_obj.write_rra(rrd_name, no_restore=False)
        
if __name__ == "__main__":
    main()
